\name{plot.cvscoop}
\alias{plot.cvscoop}
\title{Plot method for cross validated error of a scoop model}
\description{Produce a plot of the cross validated error of a \code{scoop} model.}
\usage{
\method{plot}{cvscoop}(x,
        xlab = NULL, log.scale=TRUE,
        ylab = "Mean CV error", ...)}

\arguments{
  \item{x}{output of a \code{crossval.scoop} run (must be of class
  \code{cvscoop}).}
  \item{xlab}{title for the x axis. If \code{NULL}, will be set to
  \code{"lambda"} or \code{"lambda (log-scale)"} according to \code{log.scale}.}
  \item{ylab}{title for the y axis}
  \item{log.scale}{logical; indicates if a log-scale should be used.}
  \item{\dots}{additional arguments for generic plot.}
}

\author{Julien Chiquet}
\seealso{See also \code{\link{crossval.scoop}}.}
\examples{
\dontrun{
data(basal)
attach(basal)
x <- as.matrix(expr) ## a n=29 x p=27 matrix
y <- as.numeric(status)-1 ## set pcr/not-pcr to 1/0 
group <- basal$group ## avoid global env conflict
set.seed(47)
cat("\nmight take about half a minute...")
coolasso <- scoop(x, y, group, family="binomial", n.lambda=50)
out.cv <- crossval(coolasso, K=5)
plot(out.cv)
detach(basal)}
}
\keyword{hplot}
