// Compile with :  R CMD SHLIB proximal.c
#include "scoop_utils.h"

void ISTA_LM_LAS(int *p,
		 double XtX[*p][*p],
		 double Xty[*p],
		 double lambda[*p],
		 double *L,
		 int *max_it,
		 double *eps,
		 double x0[*p],
		 double xk[*p],
		 int *i) {
  
  double delta = 2 * *eps, u[*p];
  int j,l;
  
  while ( (delta > *eps) && (*i < *max_it) ) {

    // arg1 proximal: u = x0 - (XtX %*% x0 - Xty)/L
    for (j=0; j<*p; j++) {
      u[j] = 0;
      for (l=0;l<*p; l++) {
	u[j] += XtX[j][l] * x0[l];
      }
      u[j] = x0[j] - (u[j] - Xty[j]) / *L;
    }

    proximal_las(*p, u, lambda, *L, xk);

    // preparing next iterate
    delta = 0;
    for(j=0;j<*p;j++) {
      delta += pow(x0[j]-xk[j],2);
    }
    delta = sqrt(delta);
    
    for(j=0;j<*p;j++) {
      x0[j] = xk[j];
    }
    *i += 1;

    R_CheckUserInterrupt();
  }
}

void ISTA_LM_GRP(int *p,
		 int *K,
		 int pk[*K],
		 double XtX[*p][*p],
		 double Xty[*p],
		 double lambda[*p],
		 double *L,
		 int *max_it,
		 double *eps,
		 double x0[*p],
		 double xk[*p],
		 int *i) {
  
  double delta = 2 * *eps, u[*p];
  int j,l;
  
  while ( (delta > *eps) && (*i < *max_it) ) {

    // arg1 proximal: u = x0 - (XtX %*% x0 - Xty)/L
    for (j=0; j<*p; j++) {
      u[j] = 0;
      for (l=0;l<*p; l++) {
	u[j] += XtX[j][l] * x0[l];
      }
      u[j] = x0[j] - (u[j] - Xty[j]) / *L;
    }

    proximal_grp(*p, *K, pk, u, lambda, *L, xk);

    // preparing next iterate
    delta = 0;
    for(j=0;j<*p;j++) {
      delta += pow(x0[j]-xk[j],2);
    }
    delta = sqrt(delta);
    
    for(j=0;j<*p;j++) {
      x0[j] = xk[j];
    }
    *i += 1;

    R_CheckUserInterrupt();
  }
}

void ISTA_LM_COO(int *p,
		 int *K,
		 int pk[*K],
		 double XtX[*p][*p],
		 double Xty[*p],
		 double lambda[*p],
		 double *L,
		 int *max_it,
		 double *eps,
		 double x0[*p],
		 double xk[*p],
		 int *i) {
  
  double delta = 2 * *eps, u[*p];
  int j,l;
  
  while ( (delta > *eps) && (*i < *max_it) ) {

    // arg1 proximal: u = x0 - (XtX %*% x0 - Xty)/L
    for (j=0; j<*p; j++) {
      u[j] = 0;
      for (l=0;l<*p; l++) {
	u[j] += XtX[j][l] * x0[l];
      }
      u[j] = x0[j] - (u[j] - Xty[j]) / *L;
    }

    proximal_coo(*p, *K, pk, u, lambda, *L, xk);

    // preparing next iterate
    delta = 0;
    for(j=0;j<*p;j++) {
      delta += pow(x0[j]-xk[j],2);
    }
    delta = sqrt(delta);
    
    for(j=0;j<*p;j++) {
      x0[j] = xk[j];
    }
    *i += 1;

    R_CheckUserInterrupt();
  }
}

void FISTA_LM_LAS(int *p,
		  double XtX[*p][*p],
		  double Xty[*p],
		  double lambda[*p],
		  double *L,
		  int *max_it,
		  double *eps,
		  double x0[*p],
		  double xk[*p],
		  int *i) {
  
  double delta = 2 * *eps ;
  double t0 = 1.0, tk;
  double u[*p], y0[*p], yk[*p];
  int j,l;

  // initializing y0
  for (j=0; j<*p; j++) {
    y0[j] = x0[j];
  }
  
  while ( (delta > *eps) && (*i < *max_it) ) {
        
    // arg1 proximal: u = y0 - (XtX %*% y0 - Xty)/L
    for (j=0; j<*p; j++) {
      u[j] = 0;
      for (l=0;l<*p; l++) {
	u[j] += XtX[j][l] * y0[l];
      }
      u[j] = y0[j] - (u[j] - Xty[j]) / *L;
    }

    // updating xk
    proximal_las(*p, u, lambda, *L, xk);
    // updating t
    tk = 0.5 * (1+sqrt(1+4* t0*t0)) ;
    // updating yk
    for (j=0; j<*p; j++) {
      yk[j] = xk[j] + (t0-1.0)/tk * (xk[j] - x0[j]);
    }    

    // preparing next iterate
    delta = 0;
    for(j=0;j<*p;j++) {
      delta += pow(x0[j]-xk[j],2);
    }
    delta = sqrt(delta);
    t0 = tk;
    for(j=0;j<*p;j++) {
      x0[j] = xk[j];
      y0[j] = yk[j];
    }
    *i += 1;

    R_CheckUserInterrupt();
  }
}

void FISTA_LM_GRP(int *p,
		  int *K,
		  int pk[*K],
		  double XtX[*p][*p],
		  double Xty[*p],
		  double lambda[*p],
		  double *L,
		  int *max_it,
		  double *eps,
		  double x0[*p],
		  double xk[*p],
		  int *i) {
  
  double delta = 2 * *eps ;
  double t0 = 1.0, tk;
  double u[*p], y0[*p], yk[*p];
  int j,l;

  // initializing y0
  for (j=0; j<*p; j++) {
    y0[j] = x0[j];
  }
  
  while ( (delta > *eps) && (*i < *max_it) ) {
        
    // arg1 proximal: u = y0 - (XtX %*% y0 - Xty)/L
    for (j=0; j<*p; j++) {
      u[j] = 0;
      for (l=0;l<*p; l++) {
	u[j] += XtX[j][l] * y0[l];
      }
      u[j] = y0[j] - (u[j] - Xty[j]) / *L;
    }

    // updating xk
    proximal_grp(*p, *K, pk, u, lambda, *L, xk);
    // updating t
    tk = 0.5 * (1+sqrt(1+4* t0*t0)) ;
    // updating yk
    for (j=0; j<*p; j++) {
      yk[j] = xk[j] + (t0-1.0)/tk * (xk[j] - x0[j]);
    }    

    // preparing next iterate
    delta = 0;
    for(j=0;j<*p;j++) {
      delta += pow(x0[j]-xk[j],2);
    }
    delta = sqrt(delta);
    t0 = tk;
    for(j=0;j<*p;j++) {
      x0[j] = xk[j];
      y0[j] = yk[j];
    }
    *i += 1;

    R_CheckUserInterrupt();
  }
}

void FISTA_LM_COO(int *p,
		  int *K,
		  int pk[*K],
		  double XtX[*p][*p],
		  double Xty[*p],
		  double lambda[*p],
		  double *L,
		  int *max_it,
		  double *eps,
		  double x0[*p],
		  double xk[*p],
		  int *i) {
  
  double delta = 2 * *eps ;
  double t0 = 1.0, tk;
  double u[*p], y0[*p], yk[*p];
  int j,l;

  // initializing y0
  for (j=0; j<*p; j++) {
    y0[j] = x0[j];
  }
  
  while ( (delta > *eps) && (*i < *max_it) ) {
        
    // arg1 proximal: u = y0 - (XtX %*% y0 - Xty)/L
    for (j=0; j<*p; j++) {
      u[j] = 0;
      for (l=0;l<*p; l++) {
	u[j] += XtX[j][l] * y0[l];
      }
      u[j] = y0[j] - (u[j] - Xty[j]) / *L;
    }

    // updating xk
    proximal_coo(*p, *K, pk, u, lambda, *L, xk);
    // updating t
    tk = 0.5 * (1+sqrt(1+4* t0*t0)) ;
    // updating yk
    for (j=0; j<*p; j++) {
      yk[j] = xk[j] + (t0-1.0)/tk * (xk[j] - x0[j]);
    }    

    // preparing next iterate
    delta = 0;
    for(j=0;j<*p;j++) {
      delta += pow(x0[j]-xk[j],2);
    }
    delta = sqrt(delta);
    t0 = tk;
    for(j=0;j<*p;j++) {
      x0[j] = xk[j];
      y0[j] = yk[j];
    }
    *i += 1;

    R_CheckUserInterrupt();
  }
}

/* void ISTA_LRM_LAS(int *p, */
/* 		  int *n, */
/* 		  double X[*p][*n], */
/* 		  double y[*n], */
/* 		  double lambda[*p], */
/* 		  double *L, */
/* 		  int *max_it, */
/* 		  double *eps, */
/* 		  double x0[*p], */
/* 		  double xk[*p], */
/* 		  int *i) { */
  
/*   double normx0, delta = 2 * *eps, u[*p], eta[*n]; */
/*   int j,k,l ; */
/*   double Lp; */
  
/*   //  printf("\n delta=",delta); */

/*   while ( (delta > *eps) && (*i < *max_it) ) { */

/*     // arg1 proximal: u = x0 - (-Xt(y - (1+exp(-eta))^-1)/L) */
/*     // with eta = X * x0 */
/*     for (l=0; l<*n; l++) { */
/*       eta[l] = *mu; */
/*       for (j=0;j<*p;j++) { */
/* 	eta[l] += X[j][l] * x0[j]; */
/*       } */
/*     } */
/*     for (j=0; j<*p; j++) { */
/*       u[j] = 0; */
/*       for (l=0;l<*n;l++) { */
/* 	u[j] += X[j][l] * (y[l] - 1/(1+exp(-eta[l]))); */
/*       } */
/*       u[j] = x0[j] + u[j] / *L; */
/*     } */
    
/*     proximal_las(*p, u, lambda, *L, xk); */
    
/*     // preparing next iterate */
/*     delta = 0; */
/*     for(j=0;j<*p;j++) { */
/*       delta += pow(x0[j]-xk[j],2); */
/*     } */
/*     /\**    normx0 = 0; */
/*     for(j=0;j<*p;j++) { */
/*       normx0 += pow(x0[j],2); */
/*       }**\/ */
/*       delta = sqrt(delta);///sqrt(normx0); */
/*     //    printf(" %f",delta); */
    
/*     for(j=0;j<*p;j++) { */
/*       x0[j] = xk[j]; */
/*     } */
/*     *i += 1; */

/*     R_CheckUserInterrupt(); */
/*   } */
/*   // printf("\n%i",*i); */
/* } */


