#include <R.h>
#include <Rmath.h>
#include <math.h>
#include <R_ext/Utils.h>

void groupnorm(int     *p,
	       int    *K,
	       int    pk[*K],
	       double beta[*p],
	       double norm[*K]) {
  
  int ind=0;
  
  for (int k=0; k<*K; k++) {
    for (int i=0; i<pk[k]; i++) {
      norm[k] += pow(beta[ind],2);
      ind++;
    }
    norm[k] = sqrt(norm[k]);
  }
}

void groupnormrep(int   *p,
		  int    *K,
		  int    pk[*K],
		  double beta[*p],
		  double norm[*p]) {
  
  int ind=0, ind0;
  double normk;
  
  for (int k=0; k<*K; k++) {
    ind0=ind;
    normk=0;
    for (int i=0; i<pk[k]; i++) {
      normk += pow(beta[ind],2);
      ind++;
    }
    for (int j=ind0; j<(ind0+pk[k]); j++) {
      norm[j] = sqrt(normk);
    }
  }
}

void proximal_las(int     p,
		  double u[p],
		  double lambda[p],
		  double L,
		  double res[p]) {
  
  for (int j=0; j<p; j++) {
    res[j] = fmax(0,1-lambda[j]/(L*fabs(u[j]))) * u[j];
  }
}

void proximal_grp(int     p,
		  int     K,
		  int     pk[K],
		  double u[p],
		  double lambda[p],
		  double L,
		  double res[p]) {
  
  int ind=0, ind0;
  double normk;
  
  for (int k=0; k<K; k++) {
    ind0=ind;
    normk=0;
    for (int i=0; i<pk[k]; i++) {
      normk += pow(u[ind],2);
      ind++;
    }
    for (int j=ind0; j<(ind0+pk[k]); j++) {
      res[j] = fmax(0,1-lambda[j]/(L*sqrt(normk))) * u[j];
    }
  }
}

void proximal_coo(int     p,
		  int     K,
		  int     pk[K],
		  double u[p],
		  double lambda[p],
		  double L,
		  double res[p]) {
  
  int ind=0, ind0;
  double normk, normk_pos, normk_neg;
  
  for (int k=0; k<K; k++) {
    ind0=ind;
    normk_pos=0;
    normk_neg=0;
    for (int i=0; i<pk[k]; i++) {
      if(u[ind] > 0) {
	normk_pos += pow(u[ind],2);
      } else {
	normk_neg += pow(u[ind],2);
      }
      ind++;
    }
    for (int j=ind0; j<(ind0+pk[k]); j++) {
      if(u[j] > 0) {
	res[j] = fmax(0,1-lambda[j]/(L*sqrt(normk_pos))) * u[j];
      } else {
	res[j] = fmax(0,1-lambda[j]/(L*sqrt(normk_neg))) * u[j];	
      }
    }
  }
}
